/**
 * This file demonstrates the ability to command a module.
 */

#include <stdio.h>
#include <math.h>

#include "hebi_lookup.h"
#include "hebi_command.h"
#include "hebi_mac_address.h"
#include "lookup_helpers.c"
#include "hebi_util.h"

int main(int argc, char* argv[])
{
  // Try and get the requested module.
  HebiModulePtr module = getModuleFromArgs(argc, argv);
  if (module == NULL)
  {
    hebiCleanup();
    return -1;
  }

  // Create a command object; this can be sent to the module
  HebiCommandPtr command = hebiCommandCreate();
  if (command == NULL)
  {
    printf("Could not create command object!\n");
    hebiReleaseModule(module);
    hebiCleanup();
    return -1;
  }

  // Send commands to the module in a loop.
  // Note that these packets may be dropped if network traffic is too high, so
  // be sure to close a feedback loop at the high level!
  float period = 0.5f;
  float t;
  for (t = 0.0f; t < 10.0f; t += period)
  {
    hebiCommandSetHighResAngle(command, CommandHighResAnglePosition, 0, sin(t * 0.5f));
    hebiModuleSendCommand(module, command);
    hebi_sleep_ms(period * 1000);
  }

  // For critical packets, we can verify that they were sent by requesting
  // confirmation from the module.  If the acknowledgement function returns
  // 'true', it got positive confirmation. If it returns 'false', EITHER:
  // - the sent command was dropped
  // - the sent command was received by the module, but its response was either
  //   dropped or the timeout period expired before receipt of the module.
  // Again, a high-level process should intelligently handle these conditions!
  // Note that this is a blocking call, and so for high-frequency applications,
  // SendCommand should be used instead.
  period = 0.5f;
  long timeout_ms = 100;
  for (t = 0.0f; t < 10.0f; t += period)
  {
    hebiCommandSetHighResAngle(command, CommandHighResAnglePosition, 0, sin(t * 0.5f));
    if (hebiModuleSendCommandWithAcknowledgement(module, command, timeout_ms) == 0)
    {
      printf("Got acknowledgement.\n");
    }
    else
    {
      printf("Did not receive acknowledgement!\n");
    }
    hebi_sleep_ms(period * 1000);
  }

  // We must release the resources back to the library when done!
  hebiCommandDestroy(command);
  hebiReleaseModule(module);
  hebiCleanup();
  return 0;
}
