/**
 * This file demonstrates the ability to command settings to a group.
 */

#include <stdio.h>
#include <math.h>

#include "hebi_lookup.h"
#include "hebi_group_command.h"
#include "hebi_command.h"
#include "hebi_mac_address.h"
#include "lookup_helpers.c"

int main(int argc, char* argv[])
{
  // Try and get the requested group.
  HebiGroupPtr group = getGroupFromArgs(argc, argv);
  if (group == NULL)
  {
    hebiCleanup();
    return -1;
  }

  int num_modules = hebiGroupGetNumberOfModules(group);

  // Create a command object; this can be sent to the group
  HebiGroupCommandPtr command = hebiGroupCommandCreate(num_modules);
  if (command == NULL)
  {
    printf("Could not create command object!\n");
    hebiReleaseGroup(group);
    hebiCleanup();
    return -1;
  }

  int module_index;
  long timeout_ms = 1000;
  HebiGroupInfoPtr info = hebiGroupInfoCreate(num_modules);
  if (info == NULL)
  {
    printf("Could not create info objects!\n");
    hebiGroupCommandDestroy(command);
    hebiReleaseGroup(group);
    hebiCleanup();
    return -1;
  }

  printf("Getting current spring constants.\n");
  if (hebiGroupRequestInfo(group, info, timeout_ms) == 0)
  {
    printf("Spring constants:");
    for (module_index = 0; module_index < num_modules; module_index++)
    {
      // Get the info for this particular module
      HebiInfoPtr module_info = hebiGroupInfoGetModuleInfo(info, module_index);
      // And print out the spring constants:
      if (hebiInfoHasFloat(module_info, InfoFloatSpringConstant))
        printf(" %f", hebiInfoGetFloat(module_info, InfoFloatSpringConstant));
      else
        printf(" (no data)");
    }
    printf("\n");
  }
  else
  {
    printf("Get failed!\n");
    printf("Quitting early...\n");
    hebiGroupInfoDestroy(info);
    hebiGroupCommandDestroy(command);
    hebiReleaseGroup(group);
    hebiCleanup();
    return -1;
  }

  // Set the spring constants:
  printf("Setting spring constants to 1.\n");
  float new_spring_constant = 1.0f;
  for (module_index = 0; module_index < num_modules; module_index++)
  {
    // We get the command for this particular module
    HebiCommandPtr module_cmd = hebiGroupCommandGetModuleCommand(command, module_index);
    // Set the spring constant for this module.
    // (Note -- we do not save this value, so upon module reset this will be
    // cleared)
    hebiCommandSetFloat(module_cmd, CommandFloatSpringConstant, new_spring_constant);
  }

  if (hebiGroupSendCommandWithAcknowledgement(group, command, timeout_ms) == 0)
  {
    printf("Got acknowledgement.\n");
  }
  else
  {
    printf("Did not receive acknowledgement!\n");
  }

  // Checking spring constant (use new info message so we don't lose old values)
  HebiGroupInfoPtr info_new = hebiGroupInfoCreate(num_modules);
  if (info_new != NULL)
  {
    printf("Getting new spring constants:\n");
    if (hebiGroupRequestInfo(group, info_new, timeout_ms) == 0)
    {
      printf("Spring constants:");
      for (module_index = 0; module_index < num_modules; module_index++)
      {
        // Get the info for this particular module
        HebiInfoPtr module_info = hebiGroupInfoGetModuleInfo(info_new, module_index);
        // And print out the spring constants:
        if (hebiInfoHasFloat(module_info, InfoFloatSpringConstant))
          printf(" %f", hebiInfoGetFloat(module_info, InfoFloatSpringConstant));
        else
          printf(" (no data)");
      }
      printf("\n");
    }
    else
    {
      printf("Get failed!\n");
    }
  }

  // Reset the spring constants:
  printf("Resetting spring constants to previous values.\n");
  for (module_index = 0; module_index < num_modules; module_index++)
  {
    // We get the command for this particular module
    HebiCommandPtr module_cmd = hebiGroupCommandGetModuleCommand(command, module_index);
    // Set the spring constant for this module to the previous value, or nothing
    // if we didn't get it back before (this is reading from the previously
    // received info packet)
    HebiInfoPtr module_info = hebiGroupInfoGetModuleInfo(info, module_index);
    if (hebiInfoHasFloat(module_info, InfoFloatSpringConstant))
    {
      hebiCommandSetFloat(module_cmd, CommandFloatSpringConstant,
        hebiInfoGetFloat(module_info, InfoFloatSpringConstant));
    }
    else
    {
      hebiCommandClearFloat(module_cmd, CommandFloatSpringConstant);
    }
  }

  if (hebiGroupSendCommandWithAcknowledgement(group, command, timeout_ms) == 0)
  {
    printf("Got acknowledgement.\n");
  }
  else
  {
    printf("Did not receive acknowledgement!\n\
Warning: spring constants may not be set back to previous values!\n");
  }

  // We must release the resources back to the library when done!
  hebiGroupInfoDestroy(info);
  if (info_new != NULL) hebiGroupInfoDestroy(info_new);
  hebiGroupCommandDestroy(command);
  hebiReleaseGroup(group);
  hebiCleanup();
  return 0;
}
